#!/usr/bin/env python
# coding=utf-8

"""
todotxt_machine
~~~~~~~~~~~~~~~

An interactive terminal based todo.txt file editor with an interface similar to mutt

"""

__name__ = "todotxt_machine"
__title__ = "todotxt_machine"
__author__ = "Anthony DiGirolamo"
__license__ = "GPL3"
__version__ = (2, 0, 5)

version = "%s.%s.%s" % __version__

__all__ = ["cli", "todo", "colorscheme", "urwid_ui", "terminal_operations"]
